using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace RegionEditor
{
    public class SpawnGroups
    {
        private List<SpawnGroup> m_Spawns;

        public List<SpawnGroup> Spawns
        {
            get { return m_Spawns; }
            set { m_Spawns = value; }
        }

        public SpawnGroups()
        {
            m_Spawns = new List<SpawnGroup>();
        }

        public static SpawnGroups Load( string filename )
        {
            try
            {
                XmlDocument dom = new XmlDocument();
                dom.Load( filename );

                XmlElement defs = dom["spawnDefinitions"];
                SpawnGroups sg = new SpawnGroups();

                foreach ( XmlElement grp in defs.SelectNodes( "spawnGroup" ) )
                {
                    sg.m_Spawns.Add( SpawnGroup.Load( grp ) );
                }

                return sg;
            }
            catch ( Exception err )
            {
                System.Windows.Forms.MessageBox.Show( string.Format( "An error has occurred. It has been placed on your clipboard:\n\n{0}", err.ToString() ) );
                System.Windows.Forms.Clipboard.SetDataObject( err.ToString() );
            }

            return null;
        }

        public bool Save( string filename )
        {
            try
            {
                XmlDocument dom = new XmlDocument();
                XmlDeclaration dec = dom.CreateXmlDeclaration( "1.0", "utf-8", null );
                dom.AppendChild( dec );

                XmlElement defs = dom.CreateElement( "spawnDefinitions" );

                foreach ( SpawnGroup grp in m_Spawns )
                {
                    defs.AppendChild( grp.GetXml( dom ) );
                }

                dom.AppendChild( defs );
				dom.Save( filename );
                return true;
            }
            catch ( Exception err )
            {
                System.Windows.Forms.MessageBox.Show( string.Format( "An error has occurred. It has been placed on your clipboard:\n\n{0}", err.ToString() ) );
                System.Windows.Forms.Clipboard.SetDataObject( err.ToString() );
            }

            return false;
        }
	
    }

    public class SpawnGroup : IComparable
    {
        private string m_Name = string.Empty;
        private List<string> m_Items;

        public List<string> Items
        {
            get { return m_Items; }
            set { m_Items = value; }
        }

        public string Name
        {
            get { return m_Name; }
            set { m_Name = value; }
        }

        public SpawnGroup()
        {
            m_Items = new List<string>();
        }

        public static SpawnGroup Load( XmlElement xml )
        {
            SpawnGroup g = new SpawnGroup();
            XmlSupport.ReadString( xml, "name", ref g.m_Name );

            foreach ( XmlElement obj in xml.SelectNodes( "object" ) )
            {
                string type = null;

                if ( XmlSupport.ReadString( obj, "type", ref type ) )
                    g.m_Items.Add( type );
            }

            return g;
        }

        public XmlElement GetXml( XmlDocument dom )
        {
            XmlElement xml = dom.CreateElement( "spawnGroup" );
            xml.SetAttribute( "name", m_Name );

            foreach ( string s in m_Items )
            {
                XmlElement obj = dom.CreateElement( "object" );
                obj.SetAttribute( "type", s );
                xml.AppendChild( obj );
            }

            return xml;
        }

		public override string ToString()
		{
			return m_Name;
		}

		#region IComparable Members

		public int CompareTo( object obj )
		{
			SpawnGroup g = obj as SpawnGroup;

			if ( g == null )
				return 0;

			return m_Name.CompareTo( g.m_Name );
		}

		#endregion
}
}
